/* updatablekeyiterator.vala
 *
 * Copyright (C) 2009  Didier Villevalois
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Didier 'Ptitjes Villevalois <ptitjes@free.fr>
 */

#include <glib.h>
#include <glib-object.h>


#define GEE_TYPE_ITERATOR (gee_iterator_get_type ())
#define GEE_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_ITERATOR, GeeIterator))
#define GEE_IS_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_ITERATOR))
#define GEE_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_ITERATOR, GeeIteratorIface))

typedef struct _GeeIterator GeeIterator;
typedef struct _GeeIteratorIface GeeIteratorIface;

#define GEE_TYPE_UPDATABLE_KEY_ITERATOR (gee_updatable_key_iterator_get_type ())
#define GEE_UPDATABLE_KEY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), GEE_TYPE_UPDATABLE_KEY_ITERATOR, GeeUpdatableKeyIterator))
#define GEE_IS_UPDATABLE_KEY_ITERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), GEE_TYPE_UPDATABLE_KEY_ITERATOR))
#define GEE_UPDATABLE_KEY_ITERATOR_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), GEE_TYPE_UPDATABLE_KEY_ITERATOR, GeeUpdatableKeyIteratorIface))

typedef struct _GeeUpdatableKeyIterator GeeUpdatableKeyIterator;
typedef struct _GeeUpdatableKeyIteratorIface GeeUpdatableKeyIteratorIface;

struct _GeeIteratorIface {
	GTypeInterface parent_iface;
	gboolean (*next) (GeeIterator* self);
	gboolean (*has_next) (GeeIterator* self);
	gboolean (*first) (GeeIterator* self);
	gpointer (*get) (GeeIterator* self);
	void (*remove) (GeeIterator* self);
};

struct _GeeUpdatableKeyIteratorIface {
	GTypeInterface parent_iface;
	gpointer (*get_value) (GeeUpdatableKeyIterator* self);
	void (*set_value) (GeeUpdatableKeyIterator* self, gconstpointer value);
};



GType gee_iterator_get_type (void);
GType gee_updatable_key_iterator_get_type (void);
gpointer gee_updatable_key_iterator_get_value (GeeUpdatableKeyIterator* self);
void gee_updatable_key_iterator_set_value (GeeUpdatableKeyIterator* self, gconstpointer value);



gpointer gee_updatable_key_iterator_get_value (GeeUpdatableKeyIterator* self) {
	return GEE_UPDATABLE_KEY_ITERATOR_GET_INTERFACE (self)->get_value (self);
}


void gee_updatable_key_iterator_set_value (GeeUpdatableKeyIterator* self, gconstpointer value) {
	GEE_UPDATABLE_KEY_ITERATOR_GET_INTERFACE (self)->set_value (self, value);
}


static void gee_updatable_key_iterator_base_init (GeeUpdatableKeyIteratorIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
	}
}


GType gee_updatable_key_iterator_get_type (void) {
	static GType gee_updatable_key_iterator_type_id = 0;
	if (gee_updatable_key_iterator_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (GeeUpdatableKeyIteratorIface), (GBaseInitFunc) gee_updatable_key_iterator_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		gee_updatable_key_iterator_type_id = g_type_register_static (G_TYPE_INTERFACE, "GeeUpdatableKeyIterator", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (gee_updatable_key_iterator_type_id, G_TYPE_OBJECT);
		g_type_interface_add_prerequisite (gee_updatable_key_iterator_type_id, GEE_TYPE_ITERATOR);
	}
	return gee_updatable_key_iterator_type_id;
}




